package foo.payment.plugin;

import java.math.BigDecimal;
import javax.annotation.PostConstruct;
import org.slf4j.Logger;
import org.apache.commons.lang.StringUtils;
import ru.crystals.pos.api.ext.loyal.dto.DailyReportResult;
import ru.crystals.pos.api.ext.loyal.dto.DailyReportResultCode;
import ru.crystals.pos.api.plugin.PaymentPlugin;
import ru.crystals.pos.api.plugin.payment.Payment;
import ru.crystals.pos.api.plugin.user.User;
import ru.crystals.pos.api.ui.listener.ConfirmListener;
import ru.crystals.pos.api.ui.listener.DialogListener;
import ru.crystals.pos.api.ui.listener.InputScanNumberFormListener;
import ru.crystals.pos.api.ui.listener.SumToPayFormListener;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.POSInfo;
import ru.crystals.pos.spi.ResBundle;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.feedback.Feedback;
import ru.crystals.pos.spi.plugin.payment.CancelRequest;
import ru.crystals.pos.spi.plugin.payment.InvalidPaymentException;
import ru.crystals.pos.spi.plugin.payment.PaymentRequest;
import ru.crystals.pos.spi.plugin.payment.RefundRequest;
import ru.crystals.pos.spi.receipt.ProcessedPayment;
import ru.crystals.pos.spi.receipt.Receipt;
import ru.crystals.pos.spi.ui.DialogFormParameters;
import ru.crystals.pos.spi.ui.UIForms;
import ru.crystals.pos.spi.ui.payment.SumToPayFormParameters;

@POSPlugin(id = "foo.service.payment")
public class FooPaymentPlugin implements PaymentPlugin {

    /**
     * Под этим ключом в оплатах будет хранится номер кошелька клиента, по которому
     * выполняется оплата или аннулирование.
     * @see Payment#getData()
     */
    private static final String PAYMENT_KEY_WALLET_NUMBER = "wallet.number";

    /**
     * Под этим ключом в оплатах будет храниться код авторизации, получаемый при оплате.
     * @see Payment#getData()
     */
    private static final String PAYMENT_KEY_AUTHORIZATION_CODE = "authorization.code";

    /**
     * Под этим ключом в оплатах будет храниться идентификатор клиента.
     * @see Payment#getData()
     */
    private static final String PAYMENT_KEY_CLIENT_ID = "client.id";

    /**
     * Под этим ключом в оплатах хранится идентификатор транзакции, который получается при оплате
     * электронным кошельком.
     */
    private static final String PAYMENT_KEY_TRANSACTION_NUMBER = "transaction.number";

    @Inject
    private POSInfo pos; // параметры кассы
    
    @Inject
    private IntegrationProperties properties; // настройки интеграции и плагина

    @Inject
    private UIForms ui; // формы кассы

    @Inject
    private Logger log; // логирование
    
    @Inject
    private ResBundle res; // локализованные ресурсы

    // Вызов метода, аннотированного PostConstruct доступен только для касс версией от 10.2.47.0 и выше.
    @SuppressWarnings("unused")
    @PostConstruct
    private void init() {
        log.info("Plugin {} loaded", this.getClass());
    }
    
    @Override
    public void doPayment(PaymentRequest paymentRequest) {
        inputSum(paymentRequest, paymentRequest.getReceipt().getSurchargeSum());
    }
    
    private void inputSum(PaymentRequest paymentRequest, BigDecimal defaultSum) {
        // Показываем форму ввода суммы к оплате
        SumToPayFormParameters parameters = new SumToPayFormParameters(res.getString("payment.name"), paymentRequest.getReceipt());
        parameters.setInputHint(res.getString("enter.sum.to.pay"));
        parameters.setDefaultSum(defaultSum);
        ui.getPaymentForms().showSumToPayForm(parameters, new SumToPayFormListener() {

            @Override
            public void eventCanceled() {
                // была нажата <Отмена> завершим процесс
                paymentRequest.getPaymentCallback().paymentNotCompleted();
            }

            @Override
            public void eventSumEntered(BigDecimal sumToPay) {
                sumEntered(paymentRequest, sumToPay);
            }
        });
    }
    
    private void sumEntered(PaymentRequest paymentRequest, BigDecimal sumToPay) {
        ui.getInputForms().showInputScanNumberForm(res.getString("payment.name"), res.getString("scanQR"), res.getString("wallet.number"), 16, new InputScanNumberFormListener() {

            @Override
            public void eventBarcodeScanned(String barcode) {
                remoteRequest(paymentRequest, barcode, sumToPay);
            }

            @Override
            public void eventInputComplete(String number) {
                remoteRequest(paymentRequest, number, sumToPay);
            }

            @Override
            public void eventCanceled() {
                // если нажали отмену, возвращаемся в форме ввода суммы к оплате,
                // на форме будет отображена введенная сумма 
                inputSum(paymentRequest, sumToPay);
            }
        });
    }
    

    private void remoteRequest(PaymentRequest paymentRequest, String walletNumber, BigDecimal sumToPay) {
        ui.showSpinnerForm(res.getString("connecting"));
        // читаем необходимые настройки плагина:
        String url = properties.getServiceProperties().get("foo.service.url");
        int timeout = properties.getServiceProperties().getInt("timeout", 10000);
        
        // Можно сохранять какие либо параметры необходимые для работы сервиса
        int persistCounter = properties.getServiceProperties().getInt("PersistedOperationCounter", 0);
        persistCounter++;
        properties.getServiceProperties().setInt("PersistedOperationCounter", persistCounter);
        
        // читаем необходимые настройки кассы:
        int posNumber = pos.getPOSNumber();
        int shopNumber = pos.getShopNumber();
        
        try {
            // выполнение удаленного вызова, получение ответа
            // формирование данных для callback
        	Thread.sleep(2000);
            Payment payment = new Payment();
            payment.setSum(sumToPay);
            payment.getData().put(PAYMENT_KEY_AUTHORIZATION_CODE, "385281233141592");
            payment.getData().put(PAYMENT_KEY_CLIENT_ID, "4815162342");
            
            // формируем слип для печати
            StringBuilder sb = new StringBuilder();
            sb.append(res.getString("slip.header")).append("\n")
                .append(sumToPay.toString())
                .append(res.getString("slip.rub")).append("\n")
                .append(res.getString("slip.transaction.number"))
                .append("\n")
                .append("01234567890");
            payment.getSlips().add(sb.toString());

            // Добавим в мапу реквизитов номер кошелька, которым выполнялась оплата.
            // Таким образом, в случае аннулирования чека, можно осуществить возврат средств
            // на этот же кошелек, без необходимости повторно запрашивать номер.
            payment.getData().put(PAYMENT_KEY_WALLET_NUMBER, walletNumber);
            // Также сохраним идентификатор транзакции платежа
            payment.getData().put(PAYMENT_KEY_TRANSACTION_NUMBER, "01234567890");

            // процесс оплаты успешно завершен
            paymentRequest.getPaymentCallback().paymentCompleted(payment);
            // Касса продолжает процесс оплаты и регистрации чека.
            // С этого момента отображение форм уже недопустимо.
            // Попытка отобразить форму после отправка callback приведет к исключению
            // IncorrectStateException
        } catch (Exception e) {
            ui.showErrorForm(res.getString("connection.error"), new ConfirmListener() {
                @Override
                public void eventConfirmed() {
                    paymentRequest.getPaymentCallback().paymentNotCompleted();
                }
            });
        }
    }

    @Override
    public void doPaymentCancel(CancelRequest cancelRequest) {
        ui.showSpinnerForm(res.getString("cancellation.in.progress"));
        // Получим обратно номер кошелька, который мы запросили при оплате и с которого же была оплата произведена, и другие реквизиты,
        // которые бы могли понадобиться.
        String walletNumber = cancelRequest.getPayment().getData().get(PAYMENT_KEY_WALLET_NUMBER);
        String transactionId = cancelRequest.getPayment().getData().get(PAYMENT_KEY_TRANSACTION_NUMBER);
        BigDecimal sum = cancelRequest.getPayment().getSum();

        try {
            Payment cancelledPayment = cancelRequest.getPayment();
            // Эмулируем длительную операцию аннулирования оплаты.
            Thread.sleep(2000);
            // Просто чтобы дать возможность посмотреть диалоговое окно об ошибке при аннулировании
            if(StringUtils.isBlank(walletNumber)) {
                throw new IllegalArgumentException("Wallet number cannot be empty!");
            }
            cancelledPayment.getSlips().add("Cancellation slip goes here");
            // Аннулирование прошло успешно, уведомляем кассу об этом.
            // Изменять сумму в чеке нет нужды - касса сама обрабатывает результат аннулирования.
            cancelRequest.getPaymentCallback().paymentCompleted(cancelledPayment);
        } catch (Exception ex) {
            log.error("An error has occurred during check cancellation", ex);
            // Аннулирование не удалось, предложим пользователю попробовать провести аннулирование ещё раз или
            // продолжить без подтверждения об аннулирвоании.
            DialogFormParameters dialogModel = new DialogFormParameters(
                    res.getString("cancellation.failed.dialog"),
                    res.getString("cancellation.failed.dialog.button.proceed"),
                    res.getString("cancellation.failed.dialog.button.retry")
            );
            ui.showDialogForm(dialogModel, new DialogListener() {
                @Override
                public void eventButton1pressed() {
                    // Выбран вариант аннулирования чека
                    try {
                        cancelRequest.getPaymentCallback().paymentCompleted(cancelRequest.getPayment());
                    } catch (InvalidPaymentException pex) {
                        log.error("Error during check cancellation", pex);
                        ui.showErrorForm(res.getString("cancellation.generic.error"), () -> cancelRequest.getPaymentCallback().paymentNotCompleted());
                    }
                }

                @Override
                public void eventButton2pressed() {
                    // Выбран вариант повтора попытки аннулирования
                    doPaymentCancel(cancelRequest);
                }

                @Override
                public void eventCanceled() {
                    // Пользователь может отказаться от аннулирования чека вообще.
                    cancelRequest.getPaymentCallback().paymentNotCompleted();
                }
            });
        }
    }

    @Override
    public void doRefund(RefundRequest refundRequest) {
        // Показываем форму ввода суммы к оплате
        SumToPayFormParameters parameters = new SumToPayFormParameters(res.getString("refund.name"), refundRequest.getRefundReceipt());
        parameters.setInputHint(res.getString("enter.sum.to.refund"));
        parameters.setDefaultSum(refundRequest.getSumToRefund());
        ui.getPaymentForms().showSumToPayForm(parameters, new SumToPayFormListener() {

            @Override
            public void eventCanceled() {
                // была нажата <Отмена> завершим процесс
                refundRequest.getPaymentCallback().paymentNotCompleted();
            }

            @Override
            public void eventSumEntered(BigDecimal sumToPay) {
                sumRefundEntered(refundRequest, sumToPay);
            }

			private void sumRefundEntered(RefundRequest refundRequest, BigDecimal sumToPay) {
		        ui.showSpinnerForm(res.getString("connecting"));
		        // читаем необходимые настройки плагина:
		        String url = properties.getServiceProperties().get("foo.service.url");
		        int timeout = properties.getServiceProperties().getInt("timeout", 10000);
		        
		        // читаем необходимые настройки кассы:
		        int posNumber = pos.getPOSNumber();
		        int shopNumber = pos.getShopNumber();
                        
                // Получить информацию о кассире
                User currentCashier = pos.getUser();
		        
		        try {
		            // выполнение удаленного вызова, получение ответа
		            // формирование данных для callback
		        	Thread.sleep(2000);
		            Payment payment = new Payment();
		            payment.setSum(sumToPay);
		            payment.getData().put(PAYMENT_KEY_AUTHORIZATION_CODE, "some code");
		            payment.getData().put("client.id", "some client ID");
		            
		            // формируем слип для печати
		            StringBuilder sb = new StringBuilder();
		            sb.append(res.getString("refund.slip.header")).append("\n")
		                .append(sumToPay.toString())
		                .append(res.getString("slip.rub")).append("\n")
		                .append(res.getString("slip.transaction.number")).append("\n")
		                .append("01234567890");
		            payment.getSlips().add(sb.toString());
		            
		            // процесс возврата успешно завершен
		            try {
                        refundRequest.getPaymentCallback().paymentCompleted(payment);
                    } catch (InvalidPaymentException e) {
                        log.error("Failed to commit payment", e);
                    }
		            // Касса продолжает процесс возврата и регистрации чека.
		            // С этого момента отображение форм уже недопустимо.
		            // Попытка отобразить форму после отправка callback приведет к исключению
		            // IncorrectStateException
		        } catch (Exception e) {
		            ui.showErrorForm(res.getString("connection.error"), new ConfirmListener() {
		                @Override
		                public void eventConfirmed() {
		                	refundRequest.getPaymentCallback().paymentNotCompleted();
		                }
		            });
		        }
			}
        });
    }

    @Override
    public boolean isAvailable() {
    	// если необходимые для работы настройки не заполнены
        return properties.getServiceProperties().get("foo.service.url") != null;
    }

    @Override
    public DailyReportResult createDailyReport() {
        DailyReportResult result = new DailyReportResult();
        // Просто для примера и солидности затянем выполнение на некоторое время.
        try {
            Thread.sleep(2000);
        } catch (InterruptedException iex) {
            log.error("Suddenly awoken.");
        }
        result.getPrintStrings().add(res.getString("service.name"));
        result.getPrintStrings().add("ПРОДАЖ: 100");
        result.getPrintStrings().add("ВОЗВРАТОВ: 0");
        result.getPrintStrings().add("ПРОДАЖА: 10000 руб.");
        result.getPrintStrings().add("ВОЗВРАТ: 0 руб.");
        result.getPrintStrings().add("ОБОРОТ: 10000 руб.");
        result.setResultCode(DailyReportResultCode.SUCCESS);
        return result;
    }

    @Override
    public boolean hasDailyReports() {
        // Этот плагин умеет проводить сверку итогов, потому что мы реализовали метод createDailyReport,
        // поэтому, чтобы сверка по этому плагину была доступна, следует рассказать кассе, что метод можно вызывать.
        // Для этого мы возвращаем true здесь.
        return true;
    }

    @Override
    public Feedback eventReceiptFiscalized(Receipt receipt, boolean isCancelReceipt) {
        log.trace("Receipt fiscalized event. Receipt creation date: {}", receipt.getDateCreate());
        Feedback feedback = null;
        if (receipt.getPayments().stream().map(ProcessedPayment::getPluginId).anyMatch("foo.service.payment"::equals)) {
            feedback = new Feedback("Some payload for sending to server later. IsCancel=" + isCancelReceipt);
        }
        return feedback;
    }

    @Override
    public void onRepeatSend(Feedback feedback) throws Exception {
        log.trace("Entering onRepeatSend goodsFeedback.payload={}", feedback.getPayload());
        log.trace("Leaving onRepeatSend");
    }
}
