package ru.crystals.test.plugins;

import org.slf4j.Logger;
import ru.crystals.pos.api.card.BonusBalanceEntity;
import ru.crystals.pos.api.card.CardEntity;
import ru.crystals.pos.api.plugin.CardPlugin;
import ru.crystals.pos.api.plugin.card.BonusOperationResponse;
import ru.crystals.pos.api.plugin.card.BonusOperationStatus;
import ru.crystals.pos.api.plugin.card.BonusWriteOffOperationResponse;
import ru.crystals.pos.api.plugin.card.CardInfo;
import ru.crystals.pos.api.plugin.card.CardSearchResponse;
import ru.crystals.pos.api.plugin.card.CardSearchResponseStatus;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.equipment.CustomerDisplay;
import ru.crystals.pos.spi.plugin.card.CardSearchRequest;
import ru.crystals.pos.spi.receipt.Card;
import ru.crystals.pos.spi.receipt.CardStatus;
import ru.crystals.pos.spi.receipt.CardType;
import ru.crystals.pos.spi.receipt.Receipt;
import ru.crystals.test.plugins.data.AttributeEntry;
import ru.crystals.test.plugins.data.BonusResponseStatusEnum;
import ru.crystals.test.plugins.data.CardSearchStatus;
import ru.crystals.test.plugins.data.CardTypeEnum;
import ru.crystals.test.plugins.data.ObjectFactory;
import ru.crystals.test.plugins.data.PluginResult;

import javax.annotation.PostConstruct;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

@POSPlugin(id="ru.crystals.setapi.test.card.plugin")
public class TestCardPlugin implements CardPlugin {

    @Inject
    private Logger logger;

    @Inject
    private IntegrationProperties integrationProperties;

    @Inject
    private CustomerDisplay display;

    private Unmarshaller unmarshaller;
    private ResponseReaderRoutine responseReader;
    private CustomerMessageRoutine displayRoutine = new CustomerMessageRoutine();

    @PostConstruct
    public void init() {
        JAXBContext context;
        try {
            context = JAXBContext.newInstance(ObjectFactory.class);
            unmarshaller = context.createUnmarshaller();
        } catch (JAXBException jex) {
            throw new RuntimeException("Failed to create an instance of JAXB context", jex);
        }
        responseReader = new ResponseReaderRoutine(logger, integrationProperties, unmarshaller);
        logger.info("{} started", this.getClass());
    }

    @Override
    public CardSearchResponse searchCard(CardSearchRequest request) {
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getCardPlugin() == null || pr.getCardPlugin().getCardSearch() == null) {
            return new CardSearchResponse(CardSearchResponseStatus.UNKNOWN_CARD);
        }
        displayRoutine.displayMessages(display, pr.getCardPlugin().getCardSearch().getCustomerDisplayMessage());
        CardSearchResponseStatus status = createCardSearchResponseStatus(pr.getCardPlugin().getCardSearch().getStatus());
        if(status != CardSearchResponseStatus.OK) {
            CardSearchResponse r = new CardSearchResponse(status);
            if(pr.getCardPlugin().getCardSearch().getMessage() != null) {
                r.getCashierMessages().addAll(pr.getCardPlugin().getCardSearch().getMessage());
            }
            return r;
        }
        CardSearchResponse response = new CardSearchResponse(status);
        if(pr.getCardPlugin().getCardSearch().getMessage() != null) {
            response.getCashierMessages().addAll(pr.getCardPlugin().getCardSearch().getMessage());
        }
        if(pr.getCardPlugin().getCardSearch().getCard() == null) {
            return response;
        }
        CardEntity card = new CardEntity();
        card.setId(pr.getCardPlugin().getCardSearch().getCard().getId());
        card.setCardNumber(pr.getCardPlugin().getCardSearch().getCard().getNumber());
        card.setCardStatus(createCardStatus(pr.getCardPlugin().getCardSearch().getCard().getStatus()));
        card.setCardType(createCardType(pr.getCardPlugin().getCardSearch().getCard().getType()));
        if(pr.getCardPlugin().getCardSearch().getCard().getBalance() != null) {
            BonusBalanceEntity bbe = new BonusBalanceEntity(pr.getCardPlugin().getCardSearch().getCard().getBalance().getBalance());
            bbe.setSumAllowedToChargeOff(pr.getCardPlugin().getCardSearch().getCard().getBalance().getAllowed());
            bbe.setSumToChargeOff(pr.getCardPlugin().getCardSearch().getCard().getBalance().getCharged());
            card.setBonusBalance(bbe);
        }
        if(pr.getCardPlugin().getCardSearch().getCard().getAttributes() != null) {
            for(AttributeEntry entry : pr.getCardPlugin().getCardSearch().getCard().getAttributes()) {
                card.getExtendedAttributes().put(entry.getKey(), entry.getValue());
            }
        }
        response.setCard(card);
        return response;
    }

    @Override
    public BonusWriteOffOperationResponse writeOff(Card bonusCard, BigDecimal bonusesToWriteOff, Receipt receipt) {
        PluginResult pr = responseReader.getResultBag();
        BonusWriteOffOperationResponse response = new BonusWriteOffOperationResponse();
        if(pr == null || pr.getCardPlugin() == null || pr.getCardPlugin().getWriteoff() == null) {
            response.setResponseStatus(BonusOperationStatus.OK);
            response.setTxId("0000STUB");
            return response;
        }
        displayRoutine.displayMessages(display, pr.getCardPlugin().getWriteoff().getCustomerDisplayMessage());
        response.setResponseStatus(createBonusOperationStatus(pr.getCardPlugin().getWriteoff().getStatus()));
        response.setTxId(pr.getCardPlugin().getWriteoff().getTransactionId());
        response.setOperatorMessage(pr.getCardPlugin().getWriteoff().getMessage());
        if(pr.getCardPlugin().getWriteoff().getCardAttributes() != null) {
            for(AttributeEntry entry : pr.getCardPlugin().getWriteoff().getCardAttributes()) {
                response.getCardExtendedAttributesMap().put(entry.getKey(), entry.getValue());
            }
        }
        if(pr.getCardPlugin().getWriteoff().getReceiptAttributes() != null) {
            for(AttributeEntry entry : pr.getCardPlugin().getWriteoff().getReceiptAttributes()) {
                response.getReceiptExtendedAttributesMap().put(entry.getKey(), entry.getValue());
            }
        }
        return response;
    }

    @Override
    public BonusOperationResponse rollback(String txId) {
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getCardPlugin() == null || pr.getCardPlugin().getRollback() == null) {
            return new BonusOperationResponse(BonusOperationStatus.OK, txId);
        }
        displayRoutine.displayMessages(display, pr.getCardPlugin().getRollback().getCustomerDisplayMessage());
        return new BonusOperationResponse(createBonusOperationStatus(pr.getCardPlugin().getRollback().getStatus()),
                pr.getCardPlugin().getRollback().getTransactionId());
    }

    @Override
    public CardInfo getCardInfo(CardSearchRequest request) {
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getCardPlugin() == null || pr.getCardPlugin().getCardInfo() == null) {
            return null;
        }
        displayRoutine.displayMessages(display, pr.getCardPlugin().getCardInfo().getCustomerDisplayMessage());
        CardSearchResponseStatus status = createCardSearchResponseStatus(pr.getCardPlugin().getCardInfo().getStatus());
        if(status != CardSearchResponseStatus.OK) {
            return new CardInfo(status, new HashMap<>());
        }
        Map<String, String> cardInfoData = new LinkedHashMap<>();
        if(pr.getCardPlugin().getCardInfo().getCardInfoString() != null) {
            for(AttributeEntry entry : pr.getCardPlugin().getCardInfo().getCardInfoString()) {
                cardInfoData.put(entry.getKey(), entry.getValue());
            }
        }
        CardInfo ci = new CardInfo(status, cardInfoData);
        ci.setTitle(pr.getCardPlugin().getCardInfo().getTitle());
        return ci;
    }

    private BonusOperationStatus createBonusOperationStatus(BonusResponseStatusEnum s) {
        if(s == null) {
            return BonusOperationStatus.OK;
        }
        switch (s) {
            case ERROR:
                return BonusOperationStatus.ERROR;
            case TRY_AGAIN:
                return BonusOperationStatus.TRY_AGAIN;
            case OK:
            default:
                return BonusOperationStatus.OK;
        }
    }

    private CardType createCardType(CardTypeEnum t) {
        if(t == null) {
            return CardType.EXTERNAL;
        }
        switch (t) {
            case GIFT:
                return CardType.GIFT;
            case INTERNAL:
                return CardType.INTERNAL;
            case BONUS:
                return CardType.BONUS;
            case COUPON:
                return CardType.COUPON;
            case EXTERNAL:
            default:
                return CardType.EXTERNAL;
        }
    }
    private CardStatus createCardStatus(ru.crystals.test.plugins.data.CardStatus status) {
        if(status == null) {
            return CardStatus.ACTIVE;
        }
        switch (status) {
            case USED:
                return CardStatus.USED;
            case BLOCKED:
                return CardStatus.BLOCKED;
            case INVALID:
                return CardStatus.INVALID;
            case ACTIVE:
            default:
                return CardStatus.ACTIVE;
        }
    }

    private CardSearchResponseStatus createCardSearchResponseStatus(CardSearchStatus s) {
        if(s == null) {
            return CardSearchResponseStatus.UNKNOWN_CARD;
        }
        switch (s) {
            case OK:
                return CardSearchResponseStatus.OK;
            case OFFLINE:
                return CardSearchResponseStatus.OFFLINE;
            case ERROR:
                return CardSearchResponseStatus.ERROR;
            case CARD_NOT_FOUND:
                return CardSearchResponseStatus.CARD_NOT_FOUND;
            case UNKNOWN_CARD:
            default:
                return CardSearchResponseStatus.UNKNOWN_CARD;
        }
    }
}
