package ru.crystals.pos.api.ext.loyal.dto;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

/**
 * Сущность, представляющая "абзац" на чековом слипе.<br>
 * NOTE: "абзац" - это минимальная атомарная единица, что может быть распечатана на слипе. Представляет из себя либо абзац текста, либо ШК (штрих-код).
 * Каждый "абзац" печатается на чековой ленте с новой строки.
 */
public class SlipParagraph {
    /**
     * Префикс, если с которого начинается текст {@link SlipParagraph#payload}, то ищём в нем маркеры с атрибутами текста
     * Маркер в тексте - это число, указанное в квадратных скобках, которое указывает номера шрифта и его атрибуты для части текста,
     * следующей за маркером, но до следующего маркера или конца абзаца. Например, "\\f[18]Hello [2]world!" - напечатает строку, где
     * "Hello " - жирный Шрифт 13х48, "world!" - жирный Шрифт 13х24; - для формирования этого числа можно пользоваться {@link SlipTextOptions}
     * @since 0.0.41
     */
    public static final String FORMATTED_TEXT_PREFIX = "\\f";

    /**
     * само сообщение "абзаца", что надо вывести на чековой ленте.
     */
    private String payload;

    /**
     * Изображение, которое будет печататься данным параграфом.
     * @since 0.0.41
     */
    private BufferedImage image;

    /**
     * части абзаца с указанием номера шрифта и его атрибутов {@link SlipText}, {@link SlipTextOptions}
     * @since 0.0.41
     */
    private List<SlipText> paragraphParts;

    /**
     * Способ представления содержимого этого абзаца.
     */
    private SlipParagraphType type = SlipParagraphType.TEXT;

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link SlipParagraph}
     */
    public SlipParagraph() {
        // None
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link SlipParagraph} с типом содержимого - изображение.
     * При вызове этого конструктора параграфу автоматически выставляется тип {@link SlipParagraphType#IMAGE}.<br>
     * Не все фискальные принтеры умеют распечатывать изображение, а те, что умеют, обладают рядом ограничений.
     * Таким образом, доступные к печати изображения будут разниться в зависимости от модели фискального принтера касса использует.
     * Если фискальный принтер не поддерживает печать изображений или параметры изображения выходят за рамки допустимых, оно распечатано не будет.<br>
     * Ограничения:<br>
     *     для ПИРИТ 1Ф:<br>
     *      размер растра (считается как (длина х высота) / 8): до 11 Кб (11 264 байта);<br>
     *      ширина и высота должны быть кратны 8;<br>
     *      максимальный размер изображения 512х512 точек для чековой ленты шириной 80 мм и 336х400 точек для ленты шириной 57 мм;<br>
     *    для ПИРИТ 2Ф: <br>
     *       то же, но максимальный размер растра увеличен до 22 Кб (22 525 байта).<br>
     * Изображение должно быть черно-белым.<br>
     * В случае, если изображение не удовлетворяет заданным требованиям, касса может попытаться самостоятельно подогнать его под требования,
     * что может существенно ухудшить качество изображения. Посему рекомендуется сразу передавать черно-белые изображения в формате bmp с глубиной цвета 1 бит и
     * отвечающие остальным заданным требованиям.
     * @param image изображение
     */
    public SlipParagraph(BufferedImage image) {
        this.image = image;
        this.setType(SlipParagraphType.IMAGE);
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link SlipParagraph}
     * @param type тип параграфа
     * @param payload текстовое содержимое параграфа
     */
    public SlipParagraph(SlipParagraphType type, String payload) {
        this.type = type;
        this.payload = payload;
    }

    /**
     * Возвращает текстовое содержимое параграфа.
     * @return текстовое содержание параграфа или null, если параграф не текстовый или пустой.
     */
    public String getPayload() {
        return payload;
    }

    /**
     * Устанавливает содержимое текстового параграфа. Это имеет смысл только для параграфов типа {@link SlipParagraphType#TEXT},
     * {@link SlipParagraphType#BARCODE}, {@link SlipParagraphType#QR} - тех, которые действительно могут нести текстовое содержимое.
     * Для нетекстовых параграфов, например {@link SlipParagraphType#IMAGE} установленный текст не будет иметь эффекта
     * @param payload содержимое текстового параграфа
     */
    public void setPayload(String payload) {
        this.payload = payload;
    }

    /**
     * Возвращает тип параграфа. От типа параграфа зависит как касса будет интерпретировать его содержимое - распечатать как текст, баркод, изображение, etc.
     * @return тип параграфа
     */
    public SlipParagraphType getType() {
        return type;
    }

    /**
     * Устанавливает тип параграфа. От типа параграфа зависит как касса будет интерпретировать его содержимое - распечатать как текст, баркод, изображение, etc.
     * @param type тип параграфа
     */
    public void setType(SlipParagraphType type) {
        this.type = type;
    }

    /**
     * Возвращает изображение, которое представляет данный параграф. Имеет смысл только для параграфов типа {@link SlipParagraphType#IMAGE},
     * в остальных случаях содержимое этого поля игнорируется кассой.
     * @return изображение, которое будет печататься в данном параграфе
     * @see #SlipParagraph(BufferedImage)
     * @since 0.0.41
     */
    public BufferedImage getImage() {
        return image;
    }

    /**
     * @return части абзаца с указанием номера шрифта и его атрибутов {@link SlipText}, {@link SlipTextOptions}
     * @since 0.0.41
     */
    public List<SlipText> getParagraphParts() {
        if(paragraphParts == null) {
            paragraphParts = new ArrayList<>();
        }
        return paragraphParts;
    }

    @Override
    public String toString() {
        return "SlipParagraph{" +
                "payload='" + payload + '\'' +
                ", image=" + image +
                ", paragraphParts=" + paragraphParts +
                ", type=" + type +
                '}';
    }
}
