package ru.crystals.pos.spi.receipt;

import ru.crystals.pos.api.ext.loyal.dto.AppliedAdvertisingAction;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Представление чека кассы.
 */
public interface Receipt {

    /**
     * Получить общую сумму чека.
     *
     * @return сумма чека с двумя знаками после запятой, т.е. scale=2
     */
    BigDecimal getSumWithDiscount();

    /**
     * Получить оставшуюся сумму к оплате/возврату.
     * <pre>
     *  Сумма чека минус сумма всех уже выполненных в этом чеке оплат/возвратов.
     * </pre>
     * @return сумма с двумя знаками после запятой, т.е. scale=2
     */
    BigDecimal getSurchargeSum();

    /**
     * Получить позиции данного чека.
     *
     * @return не {@code null}
     */
    List<LineItem> getLineItems();

    /**
     * Получить номер смены, к которой относится данный чек.
     *
     * @return номер смены
     */
    int getShiftNo();

    /**
     * Получить [Прогнозируемый] номер этого чека.
     * <p>
     * NOTE: действительный номер присваивается документу (чеку) после успешной фискализации - до этого момента его порядковый номер можно только <em>прогнозировать</em>.
     *
     * @return номер данного чека
     */
    int getNumber() ;

    /**
     * Получить коллекцию карт, что были применены в данном чеке.
     *
     * @return не {@code null}
     */
    Collection<Card> getCards();

    /**
     * Получить коллекцию акцизных бутылок алкогольного товара в данном чеке.
     *
     * @return не может быть {@code null}
     */
    Collection<ExciseBottle> getExciseBottles();

    /**
     * Возвращает тип чека.
     * @since 0.0.20
     * @see ReceiptType
     * @return тип чека.
     */
    ReceiptType getType();

    /**
     * В случае чека возврата, здесь может находиться оригинальный чек продажи, по которому осуществляется возврат.<br>
     * Следует отметить, не всякие чеки возврата могут иметь чек продажи, например, если осуществляется произвольный возврат
     * (возврат покупки без чека продажи, в рамках которого она была совершена).
     * @see #getType()
     * @see ReceiptType#REFUND
     * @since 0.0.20
     * @return чек возврата или null, если такого нет
     */
    Receipt getSaleReceipt();

    /**
     * Получает мапу атрибутов, которые были добавлены к чеку.<br>
     * Позволяет добавлять к чеку любые дополнительные параметры, которые затем можно использовать где-нибудь в другом месте.
     * Мапа иммутабельна, хранит только строковые типы.
     * @return мапу атрибутов, которые были добавлены к чеку или пустую мапу, если таких нет.
     * @since 0.0.22
     */
    default Map<String, String> getData() {
        return Collections.EMPTY_MAP;
    }

    /**
     * Возвращает дату создания оригинального чека.
     * @return дата создания чека
     * @since 0.0.24
     * */
    Date getDateCreate();

    /**
     * Возвращает QR-код ОФД чека.
     * @return QR-код
     * @since 0.0.46
     */
    default String getQrCode() {
        return null;
    }

    /**
     * Получить коллекцию оплаты, что были проведены/отменены в данном чеке.
     *
     * @return не {@code null}
     * @since 0.0.44
     */
    default Collection<ProcessedPayment> getPayments() {
        return Collections.EMPTY_LIST;
    }

    /**
     * Возвращает коллекцию применившихся в чеке рекламных акций.
     * @return коллекция применившихся в чеке рекламных акций или пустая коллекция, если таких нет.
     * @since 0.0.46
     */
    default Collection<AppliedAdvertisingAction> getAppliedAdvertisingActions() {
        return Collections.EMPTY_SET;
    }

    /**
     * Возвращает дату и время фискализации данного чека.
     * @return дата и время фискализации чека или null, если чек ещё не был фискализован.
     * @since 0.0.49
     */
    default Date getFiscalizationTimestamp() {
        return null;
    }

    /**
     * Получить кассовую операцию.
     * @see ru.crystals.pos.spi.receipt.CashOperation
     * @return кассовая операция
     * @since 0.0.63
     */
    default CashOperation getCashOperation() {
        return CashOperation.INCOME;
    }
}
